@@ -206,7 +206,6 @@ NodeColumns::NodeColumns(Mesh mesh, const eckit::Configuration& config):
     else {
         halo_ = mesh::Halo(mesh);
     }
-    ATLAS_ASSERT(mesh_);
     mesh::actions::build_nodes_parallel_fields(mesh_.nodes());
     mesh::actions::build_periodic_boundaries(mesh_);
 
@@ -333,7 +332,7 @@ Field NodeColumns::createField(const eckit::Configuration& config) const {
 }
 
 Field NodeColumns::createField(const Field& other, const eckit::Configuration& config) const {
-    return createField(option::datatype(other.datatype()) | option::levels(other.levels()) |
+    return createField(option::name(other.name()) | option::datatype(other.datatype()) | option::levels(other.levels()) |
                        option::variables(other.variables()) | config);
 }
 
@@ -540,7 +539,9 @@ void NodeColumns::scatter(const FieldSet& global_fieldset, FieldSet& local_field
         auto name = loc.name();
         glb.metadata().broadcast(loc.metadata(), root);
         loc.metadata().set("global", false);
-        loc.metadata().set("name", name);
+        if( !name.empty() ) {
+            loc.metadata().set("name", name);
+        }
     }
 }
 
