@@ -48,7 +48,6 @@ namespace detail {
 
 namespace {
 
-
 template <typename T, typename Field>
 array::LocalView<T, 3> make_leveled_view(Field& field) {
     using namespace array;
@@ -87,8 +86,7 @@ std::string checksum_3d_field(const parallel::Checksum& checksum, const Field& f
     return checksum.execute(surface.data(), surface_field.stride(0));
 }
 
-}  // namespace
-
+} // namespace
 
 class StructuredColumnsHaloExchangeCache : public util::Cache<std::string, parallel::HaloExchange>,
                                            public grid::detail::grid::GridObserver {
@@ -487,18 +485,23 @@ void StructuredColumns::Map2to1::resize(std::array<idx_t, 2> i_range, std::array
 }
 
 const util::PartitionPolygon& StructuredColumns::polygon(idx_t halo) const {
-    if (not polygon_) {
-        polygon_.reset(new grid::StructuredPartitionPolygon(*this, halo));
+    if (halo >= static_cast<idx_t>(polygons_.size())) {
+        polygons_.resize(halo + 1);
     }
-    return *polygon_;
+    if (not polygons_[halo]) {
+        if (halo > this->halo()) {
+            throw_Exception("StructuredColumsn does not contain a halo of size " + std::to_string(halo) + ".", Here());
+        }
+        polygons_[halo].reset(new grid::StructuredPartitionPolygon(*this, halo));
+    }
+    return *polygons_[halo];
 }
 
 const util::PartitionPolygons& StructuredColumns::polygons() const {
-    if (polygons_.size()) {
-        return polygons_;
+    if (all_polygons_.size() == 0) {
+        polygon().allGather(all_polygons_);
     }
-    polygon().allGather(polygons_);
-    return polygons_;
+    return all_polygons_;
 }
 
 // ----------------------------------------------------------------------------
