@@ -141,7 +141,7 @@ public:
 
     Field lonlat() const override { return field_xy_; }
     Field xy() const { return field_xy_; }
-    Field z() const { return vertical().z(); }
+    Field z() const { return field_z_; }
     Field partition() const { return field_partition_; }
     Field global_index() const override { return field_global_index_; }
     Field remote_index() const override {
@@ -218,10 +218,11 @@ private:  // data
     mutable util::ObjectHandle<parallel::GatherScatter> gather_scatter_;
     mutable util::ObjectHandle<parallel::Checksum> checksum_;
     mutable util::ObjectHandle<parallel::HaloExchange> halo_exchange_;
-    mutable std::unique_ptr<util::PartitionPolygon> polygon_;
-    mutable util::PartitionPolygons polygons_;
+    mutable std::vector<util::ObjectHandle<util::PartitionPolygon>> polygons_;
+    mutable util::PartitionPolygons all_polygons_;
 
     Field field_xy_;
+    Field field_z_;
     Field field_partition_;
     Field field_global_index_;
     mutable Field field_remote_index_;
@@ -314,9 +315,13 @@ private:  // data
     idx_t nb_partitions_;
 
     friend struct StructuredColumnsFortranAccess;
+    friend struct BlockStructuredColumnsFortranAccess;
     Map2to1 ij2gp_;
 
+    friend class BlockStructuredColumns;
     void setup(const grid::Distribution& distribution, const eckit::Configuration& config);
+
+    friend class BlockStructuredColumns;
 };
 
 // -------------------------------------------------------------------
