@@ -10,7 +10,6 @@
 
 #pragma once
 
-
 #include "atlas/grid/detail/distribution/DistributionFunction.h"
 
 namespace atlas {
@@ -22,7 +21,10 @@ class SerialDistribution : public DistributionFunctionT<SerialDistribution> {
 public:
     SerialDistribution(const Grid& grid);
 
-    ATLAS_ALWAYS_INLINE int function(gidx_t gidx) const { return 0; }
+    ATLAS_ALWAYS_INLINE int function(gidx_t gidx) const { return rank_; }
+
+private:
+    int rank_{0};
 };
 
 }  // namespace distribution
