@@ -54,12 +54,12 @@ static double to_deg = 180. * M_1_PI;
 }  // namespace
 
 struct Region {
-    int north;
-    int south;
+    int north {-1};
+    int south {-1};
     std::unique_ptr<array::Array> elems;
-    int ntriags;
-    int nquads;
-    int nnodes;
+    int ntriags {0};
+    int nquads {0};
+    int nnodes {0};
     std::vector<idx_t> lat_begin;
     std::vector<idx_t> lat_end;
     std::vector<idx_t> nb_lat_elems;
@@ -98,7 +98,7 @@ StructuredMeshGenerator::StructuredMeshGenerator(const eckit::Parametrisation& p
 
     bool three_dimensional;
     if (p.get("three_dimensional", three_dimensional) || p.get("3d", three_dimensional)) {
-        options.set("3d", three_dimensional);
+        options.set("three_dimensional", three_dimensional);
     }
 
     size_t nb_parts;
@@ -151,14 +151,14 @@ void StructuredMeshGenerator::configure_defaults() {
     // connects elements
     // to the first node only. Note this option will only be looked at in case
     // other option
-    // "3d"==true
+    // "three_dimensional"==true
     options.set("unique_pole", true);
 
     // This option creates elements that connect east to west at greenwich
     // meridian
     // when true, instead of creating periodic ghost-points at east boundary when
     // false
-    options.set("3d", false);
+    options.set("three_dimensional", false);
 
     // This option sets number of parts the mesh will be split in
     options.set("nb_parts", mpi::size());
@@ -266,12 +266,15 @@ void StructuredMeshGenerator::generate_region(const StructuredGrid& rg, const gr
 
     double max_angle        = options.getDouble("angle");
     bool triangulate_quads  = options.getBool("triangulate");
-    bool three_dimensional  = options.getBool("3d");
+    bool three_dimensional  = options.getBool("three_dimensional");
     bool has_north_pole     = eckit::types::is_approximately_equal(rg.y().front(), 90.);
     bool has_south_pole     = eckit::types::is_approximately_equal(rg.y().back(), -90.);
     bool unique_pole        = options.getBool("unique_pole") && three_dimensional && has_north_pole && has_south_pole;
     bool periodic_east_west = rg.periodic();
 
+    region.lat_begin.resize(rg.ny(), -1);
+    region.lat_end.resize(rg.ny(), -1);
+    region.nb_lat_elems.resize(rg.ny(), 0);
 
     idx_t lat_north = -1;
     idx_t lat_south = -1;
@@ -351,6 +354,13 @@ Find min and max latitudes used by this part.
         }
     }
 
+    if (lat_north == -1 && lat_south == -1 ) {
+        return;
+    }
+
+    ATLAS_ASSERT(lat_north >= 0);
+    ATLAS_ASSERT(lat_south < rg.ny() );
+
     std::vector<idx_t> offset(rg.ny(), 0);
 
     int n = 0;
@@ -368,9 +378,7 @@ We need to connect to next region
     if (idx_t(lat_south + 1) <= rg.ny() - 1 && rg.nx(lat_south + 1) > 0) {
         ++lat_south;
     }
-    region.lat_begin.resize(rg.ny(), -1);
-    region.lat_end.resize(rg.ny(), -1);
-    region.nb_lat_elems.resize(rg.ny(), 0);
+
     region.north = lat_north;
     region.south = lat_south;
 
@@ -874,7 +882,7 @@ void StructuredMeshGenerator::generate_mesh(const StructuredGrid& rg, const grid
     bool mypart_at_north = std::any_of(part_north.begin(), part_north.end(), [&](int p) { return p == mypart; });
     bool mypart_at_south = std::any_of(part_south.begin(), part_south.end(), [&](int p) { return p == mypart; });
 
-    bool three_dimensional             = options.getBool("3d");
+    bool three_dimensional             = options.getBool("three_dimensional");
     bool periodic_east_west            = rg.periodic();
     bool include_periodic_ghost_points = periodic_east_west && !three_dimensional;
     bool remove_periodic_ghost_points  = periodic_east_west && three_dimensional;
@@ -993,8 +1001,11 @@ void StructuredMeshGenerator::generate_mesh(const StructuredGrid& rg, const grid
     auto flags   = array::make_view<int, 1>(nodes.flags());
     auto halo    = array::make_view<int, 1>(nodes.halo());
 
+    idx_t jnorth = -1;
+    idx_t jsouth = -1;
 
     std::vector<idx_t> node_numbering(node_numbering_size, -1);
+    if (nnodes > 0) {
     if (options.getBool("ghost_at_end")) {
         std::vector<GhostNode> ghost_nodes;
         ghost_nodes.reserve(nnodes);
@@ -1165,7 +1176,6 @@ void StructuredMeshGenerator::generate_mesh(const StructuredGrid& rg, const grid
         }
     }
 
-    idx_t jnorth = -1;
     if (include_north_pole) {
         idx_t inode   = node_numbering.at(jnode);
         jnorth        = jnode;
@@ -1189,7 +1199,6 @@ void StructuredMeshGenerator::generate_mesh(const StructuredGrid& rg, const grid
         ++jnode;
     }
 
-    idx_t jsouth = -1;
     if (include_south_pole) {
         idx_t inode   = node_numbering.at(jnode);
         jsouth        = jnode;
@@ -1212,6 +1221,7 @@ void StructuredMeshGenerator::generate_mesh(const StructuredGrid& rg, const grid
         Topology::set(flags(inode), Topology::SOUTH);
         ++jnode;
     }
+    }
 
     mesh.metadata().set<size_t>("nb_nodes_including_halo[0]", nodes.size());
     nodes.metadata().set<size_t>("NbRealPts", size_t(nnodes - nnewnodes));
@@ -1263,11 +1273,12 @@ void StructuredMeshGenerator::generate_mesh(const StructuredGrid& rg, const grid
         nodes[1] = tmp;
     };
 
-    bool regular_cells_glb_idx = atlas::RegularLonLatGrid(rg);
-    if( options.getBool("triangulate") || y_numbering > 0) {
+    bool regular_cells_glb_idx = rg.regular();
+    if (options.getBool("triangulate") || y_numbering > 0 || rg.y().front() != 90. || rg.y().back() != -90.) {
         regular_cells_glb_idx = false;
     }
 
+    if ((region.nquads + region.ntriags) > 0) {
     for (idx_t jlat = region.north; jlat < region.south; ++jlat) {
         idx_t ilat  = jlat - region.north;
         idx_t jlatN = jlat;
@@ -1568,7 +1579,12 @@ void StructuredMeshGenerator::generate_mesh(const StructuredGrid& rg, const grid
             }
         }
     }
-    if (not regular_cells_glb_idx) {
+    }
+    if (regular_cells_glb_idx) {
+        mesh.cells().global_index().metadata().set("min", 1);
+        mesh.cells().global_index().metadata().set("max", rg.nx(0) * (rg.ny()-1) );
+    }
+    else {
         generateGlobalElementNumbering(mesh);
     }
 }
